<?php

namespace Drupal\locker\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

class lockerSettingsForm extends ConfigFormBase {

  public function getFormId() {
    return 'locker_admin_settings';
  }

  protected function getEditableConfigNames() {
    return ['locker.settings'];
  }

    public function buildForm(array $form, FormStateInterface $form_state) {

      //getConfig Readonly
      $config = $this->config('locker.settings'); //$config = \Drupal::config('locker.settings');

      $active_value = $config->get('locker_site_locked', NULL);
      if (empty($active_value)) {
        $active_value = 'no';
      }
      $active = array('no' => t('No'), 'yes' => t('Yes'));
      $form['locker_active'] = array(
        '#type' => 'radios',
        '#default_value' => $active_value,
        '#title' => t('Lock your Drupal site'),
        '#options' => $active,
      );

      $locker_access_options = $config->get('locker_access_options', 'user_pass');
      if(!$locker_access_options)
        $locker_access_options = 'user_pass';
      $form['locker_access_options'] = array(
        '#type' => 'radios',
        '#default_value' => $locker_access_options,
        '#title' => t('Unlock options'),
        '#options' => array('passphrase' => t('Passphrase only'), 'user_pass' => t('Username/Password'), 'roles' => t('User with one of the role')),
        '#states' => array(
          'invisible' => array(
            ':input[name="locker_active"]' => array('value' => 'no'),
          ),
        ),
      );

      $url = $config->get('locker_custom_url', NULL);
      if(strlen($url)<1) {
        $url = 'unlock.html';
      }
      $form['custom_url']['url'] = array(
        '#type' => 'textfield',
        '#default_value' => $url,
        '#title' => t('Custom URL (Default: unlock.html)'),
        '#description' => t('i.e. coming-soon.html, auth/unlock'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $form['passphrase']['passphrase'] = array(
        '#type' => 'password',
        '#title' => t('Passphrase'),
        '#description' => t('Enter your desired passphrase to unlock the site.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'passphrase'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $form['passphrase']['passphrase_confirm'] = array(
        '#type' => 'password',
        '#title' => t('Confirm passphrase'),
        '#description' => t('Confirm your passphrase.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'passphrase'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $active_username = $config->get('locker_user', NULL);
      $form['user_pass']['username'] = array(
        '#type' => 'textfield',
        '#title' => t('Username'),
        '#default_value' => $active_username,
        '#description' => t('Enter your desired username to unlock the site.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'user_pass'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $form['user_pass']['password'] = array(
        '#type' => 'password',
        '#title' => t('Password'),
        '#description' => t('Enter your desired password to unlock the site.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'user_pass'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $form['user_pass']['password_confirm'] = array(
        '#type' => 'password',
        '#title' => t('Confirm password'),
        '#description' => t('Confirm your password.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'user_pass'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $default = $config->get('roles');
      $roles = Role::loadMultiple();
      foreach ($roles as $role){
        if(in_array($role->id(),['anonymous'])) continue;
        $options_roles[$role->id()]=$role->label();
      }

      $form['roles']['roles'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Roles'),
        '#default_value' => $default,
        '#options' => $options_roles,
        '#description' => t('Choose the user roles that should be user for authentication.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'roles'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      $default = $config->get('roles_autologin');
      $form['roles']['roles_autologin'] = array(
        '#type' => 'checkboxes',
        //'#title' => t('Auto login'),
        '#default_value' => $default,
        '#options' => array('auto_login'=>'Auto login'),
        '#description' => t('Automaticlly login to site if login credentials are used.'),
        '#states' => array(
          'visible' => array(
            ':input[name="locker_access_options"]' => array('value' => 'roles'),
            ':input[name="locker_active"]' => array('value' => 'yes'),
          ),
        ),
      );

      return parent::buildForm($form, $form_state);
    }

  /**
   * Lock form validate.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $locker_active = $form_state->getValue('locker_active');

    if ($locker_active == 'yes') {
      $locker_access_options = $form_state->getValue('locker_access_options');

      if ($locker_access_options == 'user_pass') {
        $user_validate = $form_state->getValue('username');
        $pass_validate = $form_state->getValue('password');
        $pass_confirm_validate = $form_state->getValue('password_confirm');
        if (strlen($user_validate) < 1) {
            $form_state->setErrorByName('username', t('To lock your Drupal site insert a username.'));
        }
        if (strlen($pass_validate) < 1) {
            $form_state->setErrorByName('password', t('To lock your Drupal site insert a password.'));
        }
        elseif ($pass_validate != $pass_confirm_validate) {
            $form_state->setErrorByName('password_confirm', t('Passwords do not match.'));
        }
      }
      else if ($locker_access_options == 'passphrase') {
        $passphrase_validate = $form_state->getValue('passphrase');
        $passphrase_confirm_validate = $form_state->getValue('passphrase_confirm');

        if (strlen($passphrase_validate) < 1) {
          $form_state->setErrorByName('passphrase', t('To lock your Drupal site insert a passphrase.'));
        }
        elseif ($passphrase_validate != $passphrase_confirm_validate) {
          $form_state->setErrorByName('passphrase_confirm', t('Passphrases do not match.'));
        }
      }
      else if ($locker_access_options == 'roles') {
        $roles_validate = $form_state->getValue('roles');

        $rolesOne = false;
        foreach($roles_validate as $val){
          if($val) $rolesOne = true;
        }
        if(!$rolesOne) {
          $form_state->setErrorByName('roles', t('Select at least one role.'));
        }
      }

      $url = $form_state->getValue('url');
      if(!$url || substr($url,0,1)=='/' || substr($url,-1)=='/' || !preg_match("@^([a-zA-Z0-9.\-_/\?=]{1,300})$@", $url)) {
        $form_state->setErrorByName('url', t('Problem with url.'));
      }
    }
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    //getConfig Read/Write
    $config = \Drupal::service('config.factory')->getEditable('locker.settings');
    $active_radio = $form_state->getValue('locker_active');
    if ($active_radio == 'yes') {
      $locker_access_options = $form_state->getValue('locker_access_options');
      $config->set('locker_access_options', $locker_access_options);
      $locker_url = $form_state->getValue('url');
      $config->set('locker_custom_url', $locker_url);
      if ($locker_access_options == 'user_pass') {
        $user = $form_state->getValue('username');
        $pass = $form_state->getValue('password');
        $passmd5 = md5($pass);
        $config
          ->set('locker_user', $user)
          ->set('locker_password', $passmd5);
      } elseif($locker_access_options == 'passphrase'){
        $passphrase = $form_state->getValue('passphrase');
        $config
          ->set('locker_passphrase', md5($passphrase));
      } elseif($locker_access_options == 'roles'){
        $roles = $form_state->getValue('roles');
        $roles_autologin = $form_state->getValue('roles_autologin');
        $config
          ->set('roles', $roles)
          ->set('roles_autologin', $roles_autologin);
      }
      $config->set('locker_site_locked', $active_radio)->save();
      \Drupal::service("router.builder")->rebuild();
      unset($_SESSION['locker_unlocked']);
    }
    elseif ($active_radio == 'no') {
      $config->delete();
      \Drupal::service("router.builder")->rebuild();
      unset($_SESSION['locker_unlocked']);
      drupal_set_message(
         t('Successfully unlocked.')
        ,'status'
      );
    }

    parent::submitForm($form, $form_state);
  }
}
